package be.iminds.ilabt.jfed.experimenter_gui.editor;

import be.iminds.ilabt.jfed.rspec_fx.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.parser.RspecCompare;
import be.iminds.ilabt.jfed.rspec.parser.RspecParseException;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.xml.stream.XMLStreamException;
import java.io.File;

/**
 * User: twalcari
 * Date: 5/20/2015
 * Time: 09:20
 */
public class EditableRspec {
    private static final Logger LOG = LoggerFactory.getLogger(EditableRspec.class);

    /**
     * The file from which this Rspec has been loaded, or where it has been saved to the last time
     */
    @Nullable
    private File file = null;

    /**
     * The name of the experiment. Typically relates to the filename of the rspec
     */
    private final StringProperty name;

    private final ReadOnlyObjectWrapper<RequestRspecSource> requestRspecSource;

    @Nullable
    private String lastSavedRequestRspec = null;

    public EditableRspec(@Nonnull RequestRspecSource requestRspecSource,
                         @Nonnull String name,
                         @Nullable File inputFile) {
        assert requestRspecSource != null;
        assert name != null;
        this.requestRspecSource = new ReadOnlyObjectWrapper<>(requestRspecSource);
        this.name = new SimpleStringProperty(name);

        setFile(inputFile); //this setter also sets lastSavedRequestSpec
    }

    /**
     * Creates an empty editable rspec
     */
    public EditableRspec(@Nonnull String name) {
        assert name != null;
        this.requestRspecSource = new ReadOnlyObjectWrapper<>(new RequestRspecSource(new FXModelRspec("request")));
        this.name = new SimpleStringProperty(name);

        //by setting an empty rspec as "last saved", the isDirty()-function will return false if the user doesn't do
        //anything in the editor at all.
        lastSavedRequestRspec = requestRspecSource.get().getRspecXmlString();
    }

    @Nullable
    public File getFile() {
        return file;
    }

    public void setFile(@Nullable File file) {
        this.file = file;
        this.lastSavedRequestRspec = requestRspecSource.get().getRspecXmlString();
    }

    @Nonnull
    public String getName() {
        return name.get();
    }

    public void setName(@Nonnull String name) {
        this.name.set(name);
    }

    @Nonnull
    public StringProperty nameProperty() {
        return name;
    }

    @Nonnull
    public RequestRspecSource getRequestRspecSource() {
        return requestRspecSource.get();
    }

    @Nonnull
    public ReadOnlyObjectProperty<RequestRspecSource> requestRspecSourceProperty() {
        return requestRspecSource.getReadOnlyProperty();
    }

    public void setRequestRspecSource(@Nonnull RequestRspecSource requestRspecSource) {
        this.requestRspecSource.set(requestRspecSource);
    }

    public boolean isDirty() {
        if(lastSavedRequestRspec == null)
            return false;

        if (requestRspecSource.get() == null)
            return true;
        String rspecString = requestRspecSource.get().getRspecXmlString();
        if (rspecString == null)
            return true;

        try {
            RspecCompare compare =
                    new RspecCompare(rspecString, lastSavedRequestRspec);

            return !compare.isEqual();
        } catch (XMLStreamException | RspecParseException e) {
            LOG.warn("Could not properly parse Rspecs to compare for dirty detection", e);
            return true;
        }
    }
}
